// Select the SVG element and define its dimensions
const svg = d3.select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Load data from CSV
d3.csv(
  'https://gist.githubusercontent.com/Mahesh-Polisetty/d637a9f25b9e56b01f53a83258826b6e/raw/2cc8d60a0e484d82116afad1bd57d829b324a048/82.csv',
).then(function (data) {
  // Filter data to include only movies
  const movies = data.filter(
    (item) => item.type === 'Movie',
  );

  // Group movies by genre
  const genres = {};
  movies.forEach(function (movie) {
    const genre = movie.listed_in.split(',')[0].trim();
    if (!genres[genre]) {
      genres[genre] = [];
    }
    if (genres[genre].length < 3) {
      genres[genre].push(movie.title);
    }
  });

  // Structure the data for the graph
  const nodes = [
    { id: 'Movies', type: 'Category' },
    { id: 'Horror Movies', type: 'Genre' },
    { id: 'Thrillers', type: 'Genre' },
    { id: 'Romantic Movies', type: 'Genre' },
  ];

  const links = [
    { source: 'Movies', target: 'Horror Movies' },
    { source: 'Movies', target: 'Thrillers' },
    { source: 'Movies', target: 'Romantic Movies' },
  ];

  // Add nodes and links for each genre
  Object.keys(genres).forEach((genre) => {
    genres[genre].forEach((title) => {
      nodes.push({ id: title, type: 'Title' });
      links.push({ source: genre, target: title });
    });
  });

  // Create the color scale for node types
  const colorScale = d3
    .scaleOrdinal()
    .domain(['Category', 'Genre', 'Title'])
    .range(['#66c2a5', '#fc8d62', '#8da0eb']);

  // Create the force simulation
  const simulation = d3
    .forceSimulation(nodes)
    .force(
      'link',
      d3
        .forceLink(links)
        .id((d) => d.id)
        .distance(120),
    )
    .force('charge', d3.forceManyBody().strength(-500))
    .force('center', d3.forceCenter(width / 2, height / 2));

  // Append links to SVG
  const link = svg
    .append('g')
    .selectAll('line')
    .data(links)
    .enter()
    .append('line')
    .attr('class', 'link')
    .style('stroke-width', 2)
    .attr('marker-end', 'url(#arrow)');

  // Create nodes
  const node = svg
    .selectAll('.node')
    .data(nodes)
    .enter()
    .append('circle')
    .attr('class', 'node')
    .attr('r', (d) => (d.type === 'Category' ? 30 : 25))
    .style('fill', (d) => colorScale(d.type));

  // Add node labels
  const nodeLabels = svg
    .selectAll('.node-label')
    .data(nodes)
    .enter()
    .append('text')
    .attr('class', 'node-label')
    .text((d) => d.id);

  // Add title
  svg
    .append('text')
    .attr('x', width / 4)
    .attr('y', 50)
    .attr('text-anchor', 'middle')
    .attr('font-size', '24px')
    .text('Network Graph Visualization');

  // Add simulation tick function
  simulation.on('tick', () => {
    link
      .attr('x1', (d) => d.source.x)
      .attr('y1', (d) => d.source.y)
      .attr('x2', (d) => d.target.x)
      .attr('y2', (d) => d.target.y);

    node.attr('cx', (d) => d.x).attr('cy', (d) => d.y);

    nodeLabels.attr('x', (d) => d.x).attr('y', (d) => d.y);
  });
});
